# -*- encoding: utf-8 -*-

import os
from hashlib import md5
import traceback

import shutil



def exists_file(filePath):
    '''
        判断文件是否存在
        filePath:目录路径
    '''
    return os.path.exists(filePath)


def is_file(path):
    '''
        返回路径是否为文件
    '''
    return os.path.isfile(path)


def remove_file(filePath):
    '''
        删除文件
    '''
    os.remove(filePath)


def remove_folder(path):
    '''
        删除目录
    '''
    # os.rmdir(path)
    shutil.rmtree(path)


def make_folder(path):
    '''
        创建文件夹
    '''
    if exists_file(path):
        return
    os.makedir(path)


def make_folders(path):
    '''
        递归创建文件夹
    '''
    if exists_file(path):
        return
    os.makedirs(path)


def mkdirs(path, isFolder):
    '''
        递归创建文件夹
    '''
    if isFolder:
        fPath = path
    else:
        fPath = os.path.dirname(path)
    if not os.path.exists(fPath):
        os.makedirs(fPath)


def move(src, dst):
    '''
        给文件或文件夹改名（可以改路径，但是不能覆盖目标文件）
    '''
    # os.rename(src,dst)
    shutil.move(src, dst)


def walk(path):
    '''
        列举path下的所有文件、文件夹
    '''
    return os.walk(path)


def walk2(path):
    '''
        列举path下的所有文件、文件夹
    '''
    fpaths = []
    for pt, fl, fi in os.walk(path):
        for f in fi:
            p = os.path.join(pt, f)
            fpaths.append((pt, f))
    return fpaths


def get_folder_son(path):
    '''
        列举path下一层的所有文件、文件夹
    '''
    paths = os.listdir(path)
    pps = []
    for s in paths:
        p = '%s%s%s' % (path, os.sep, s)
        pps.append(p)
    return pps


def read_all_file(filePath, method='r'):
    '''
        读取所有文件，一次性读取所有内容，文件不存在返回None
        filePath：文件路径
        method：读取方式，'r'读取，'rb' 二进制方式读取
    '''
    if not exists_file(filePath=filePath):
        return None
    fh = open(filePath, method)
    try:
        c = fh.read()
        return c
    finally:
        fh.close()


def read_all_lines_file(filePath, method='r'):
    '''
        读取所有文件，一次性读取所有内容， 文件不存在返回None
        filePath：文件路径
        method：读取方式，'r'读取，'rb' 二进制方式读取
    '''
    if not exists_file(filePath=filePath):
        return None
    fh = open(filePath, method)
    try:
        c = fh.readlines()
        return c
    finally:
        fh.close()


def read_line_file(filePath, method='r', callBack=None):
    '''
        读取所有文件，一次性读取所有内容， 文件不存在返回None
        filePath：文件路径
        method：读取方式，'r'读取，'rb' 二进制方式读取
        callBack：每行读取的回调函数，有两个传入参数，行索引和行内容
    '''
    if callBack == None:
        return
    if not exists_file(filePath=filePath):
        return

    fh = open(filePath, method)
    try:
        limit = 10000
        lineIndex = 0
        while True:
            lines = fh.readlines(limit)
            if not lines:
                break
            for line in lines:
                callBack(lineIndex, line)
                lineIndex += 1
    finally:
        fh.close()


# while 1:
#     lines = file.readlines(100000)
#     if not lines:
#         break
#     for line in lines:
#         pass # do something

def write_file(filePath, content, method='w', encoding='utf-8'):
    '''
        写文件
        filePath：文件路径
        content：文件内容
        method：写入方式，'w'覆盖写，'a' 续写，'wb' 二进制覆盖写
        encoding: 字符编码,默认 utf-8
    '''
    mkdirs(filePath, False)
    fh = open(filePath, method, encoding=encoding)

    try:
        fh.write(content)
    except:
        print(traceback.format_exc())
    finally:
        fh.close()


def get_file_folder_path(path):
    '''
        获取文件所存放文件夹：
        path：文件路径，可使用__file__
    '''
    path = os.path.split(os.path.realpath(path))[0]
    return path


def get_file_folder(path):
    '''
        获取文件所存放文件夹：
        path：文件路径，可使用__file__
    '''
    return os.path.dirname(path)


def get_file_name(path):
    '''
        获取文件名：
        path：文件路径，可使用__file__
    '''
    return os.path.basename(path)

def get_file_name_no_suffix(path):
    '''
        获取无后缀文件名：
        path：文件路径，可使用__file__
    '''
    return os.path.splitext(os.path.basename(path))[0]

def get_url_file_name(path):
    '''
        获取文件名：
        path：文件路径，可使用__file__
    '''
    return os.path.basename(path.split('?')[0])


def get_file_suffix(path):
    '''
        获取文件后缀
        path：文件路径，可使用__file__
    '''
    return os.path.splitext(path)[1][1:]


def get_folder_sep():
    '''
        获取操作系统文件路径分隔符
    '''
    return os.sep


def get_line_sep():
    '''
        获取操作系统换行分隔符
    '''
    return os.linesep


def get_path_sep():
    '''
        获取操作系统目录分隔符，windows中是 ;，PATH里会用到
    '''
    return os.pathsep


def get_ext_sep():
    '''
        获取操作系统扩展名分隔符，windows中是 .
    '''
    return os.extsep


def md5_file(path):
    m = md5()
    a_file = open(path, 'rb')  # 需要使用二进制格式读取文件内容
    try:
        m.update(a_file.read())
        return m.hexdigest()
    except:
        print(traceback.format_exc())
    finally:
        a_file.close()


def get_folder_sizes(path):
    ''' 获取目录下子文件、目录的大小 '''
    cmd = 'du -sb %s/*' % (path)

    pss = os.popen(cmd).readlines()

    fileSizes = {}
    for l in pss:
        ll = l.strip().split('\t')
        size = int(ll[0])
        code = ll[1].replace(path + os.sep, '')
        fileSizes[code] = size

    return fileSizes


def get_filesize(filePath):
    fsize = os.path.getsize(filePath)
    return fsize


def get_file_md5(path):
    ''' 获取文件md5 '''
    cmd = 'md5sum %s ' % (path)
    md5s = os.popen(cmd).readlines()
    for m in md5s:
        l = m.strip()
        ls = l.split('  ')
        if len(ls) != 2:
            continue

        return ls[0].strip()




#-*- encoding: utf-8 -*-


import openpyxl
#workbook相关
from openpyxl.workbook import Workbook
#封装了很强大的excel写的功能
from openpyxl.writer.excel import ExcelWriter
#一个数字转为列字母的方法
# from openpyxl.cell import get_column_letter


def open_excel(filePath, readOnly = False):
    '''
        打开excel文件
        filePath:excel文件路径
    '''
    try:
        data = openpyxl.load_workbook(filePath, read_only = readOnly)
        return data
    except Exception as e:
        print(str(e))
        return None


def get_active_sheet(wb):
    '''
        返回当前活动的sheet
        wb：excel的workbook对象
    '''
    return wb.active


def get_sheet_by_name(wb, sheetName):
    '''
        返回标题匹配的sheet
        wb：excel的workbook对象
        sheetName：sheet的标题
    '''
    return wb.get_sheet_by_name(sheetName)

def get_sheet_names(wb):
    '''
        返回sheet的标题数组
    '''
    return wb.sheetnames


def get_cell_val(ws, row, col):
    '''
        返回sheet的单元格的值
        ws:sheet对象
        row:行索引，0开始
        col:列索引，0开始
    '''
    return ws.cell(row = row, column = col).value


def get_column_code_by_index(index):
    '''
    根据索引编号，获取excel列编号
    :param index: 索引编号，从1开始
    :return:
    '''
    if index > 16384 or index < 1:
        # 超过excel最大列数，或者小于最小索引
        return ''
    charSize = 26
    indexName = ''
    for i in range(0, 5):
        tmp = index % charSize
        index = (int)((index - 1) / charSize)
        if tmp == 0:
            tmp = charSize

        indexName = chr(64+tmp) + indexName
        if index <= 0:
            break
    return indexName








def create_excel():
    '''
        创建excel文件
    '''
    try:
        wb = Workbook()
        return wb
    except Exception as e:
        print(str(e))
        return None

def create_sheet(wb, title = '', index = None):
    '''
        创建sheet工作表
        wb：excel的workbook对象
        title：工作表标题
        index：工作表索引，从0开始，不传默认加到最后
    '''
    try:
        ws = None
        if None == index:
            ws = wb.create_sheet(title = '')
        else:
            ws = wb.create_sheet(index = index, title = '')
        return ws
    except Exception as e:
        print(str(e))
        return None


def set_sheet_title(ws, title = ''):
    '''
        设置sheet工作表标题
        ws：excel的sheet对象
        title：工作表标题
    '''
    ws.title = title


def set_cell_val(ws, row, col, val):
    '''
        返回sheet的单元格的值
        ws:sheet对象
        row:行索引，0开始
        col:列索引，0开始
        val: 单元格的值
    '''
    c = ws.cell(row = row, column = col)
    c.value = val


def save(filePath, wb):
    '''
        保存excel
        filePath：保存文件路径
        wb：excel的workbook对象
    '''
    wb.save(filePath)



if __name__ == '__main__':
    # wb = create_excel()
    # ws = get_active_sheet(wb)
    # for i in range(0, 10):
    #     for j in range(0, 12):
    #         set_cell_val(ws, i, j, str(i)+'--'+str(j))
    # save('C:\\Users\\Administrator\\Desktop\\a.xlsx', wb)
    wb = open_excel('../../test/excel/demo1.xlsx', True)
    print(get_sheet_names(wb))
    # for i in tuple(get_active_sheet(wb).rows):
    #     print(i)
    # for i in tuple(get_active_sheet(wb).columns):
    #     print(i)

    for row in get_active_sheet(wb).values:
        print("--")
        for value in row:
            print(value)



#-*- encoding: utf-8 -*-

import os
import sys

import argparse
import pyperclip


from soc_excel_convert._utils import excel_utils, file_utils, str_utils
from soc_excel_convert._internal.result_code import ResultCode




def print_msg(msg):
    print(str_utils.json_encode(msg))


def read_args():
    ''' read command line args '''
    parser = argparse.ArgumentParser()
    parser.add_argument("-e", "--excel", help="Excel file path. If it is empty, it will get excel data from the clipboard by default.")
    parser.add_argument("-m", "--mode", help = "Convert format.support:md (markdown); default md.", action="store_true", default = "md")
    parser.add_argument("-t", "--target", help="Target file path. Default ./{FileName}/ ", action="store_true", default = "")
    parser.add_argument("-s", "--sheets", help="Convert excel sheet name. Default all sheets. Multiple comma separated.", action="store_true",
                        default="")
    parser.add_argument("-c", "--copy", help = "Output clipboard contents.Default not output.", action="store_true", default = "")

    return parser.parse_args()




'''
content内容结构：

{
    "sheetName": {
        "maxBeginIndex": 1,
        "maxEndIndex": 3,
        "title": {
            "beginIndex": 1,
            "endIndex": 3,
            "data": [
                {
                    "content": "name1",
                    "horizontal": "left",
                    "link": "http://xxx.xxx.xxx/"
                },
                {
                    "content": "name2",
                    "horizontal": "middle",
                    "link": "http://xxx.xxx.xxx/"
                },
                {
                    "content": "name3",
                    "horizontal": "right",
                    "link": "http://xxx.xxx.xxx/"
                }
            ]
        },
        "data": [
            {
                "beginIndex": 1,
                "endIndex": 3,
                "data": [
                    {
                        "content": "name1",
                        "link": "http://xxx.xxx.xxx/"
                    },
                    {
                        "content": "name2",
                        "link": "http://xxx.xxx.xxx/"
                    },
                    {
                        "content": "name3",
                        "link": "http://xxx.xxx.xxx/"
                    }
                ]
            },
            {
                "beginIndex": 1,
                "endIndex": 3,
                "data": [
                    {
                        "content": "name1",
                        "link": "http://xxx.xxx.xxx/"
                    },
                    {
                        "content": "name2",
                        "link": "http://xxx.xxx.xxx/"
                    },
                    {
                        "content": "name3",
                        "link": "http://xxx.xxx.xxx/"
                    }
                ]
            }
        ]
    }
}
'''


def read_excel(excelPath, sheets):
    ''' 读取excel内容 '''
    wb = excel_utils.open_excel(excelPath)

    sheetNames = excel_utils.get_sheet_names(wb)

    contents = {}

    for name in sheetNames:
        if len(sheets) > 0 and name not in sheets:
            continue
        contents[name] = read_excel_content(wb, name)
    return contents


def read_excel_content(wb, sheetName):
    ''' 读取excel具体内容 '''
    ws = excel_utils.get_sheet_by_name(wb, sheetName)

    rows = ws.values

    rowIndex = 0
    content = {
        "maxBeginIndex": 0,
        "maxEndIndex": 0,
        "title": None,
        "data": []
    }

    for rowInfo in rows:
        rowIndex += 1
        row = {
            "beginIndex": 0,
            "endIndex": 0,
            "data": []
        }
        colIndex = 0
        for value in rowInfo:
            colIndex += 1

            if row['beginIndex'] < 1 and (value is None or value == ""):
                continue

            row['beginIndex'] = colIndex if row['beginIndex'] < 1 else row['beginIndex']
            row['endIndex'] = colIndex if (value is not None and value != "") else row['endIndex']

            if value is None:
                value = ""

            columnName = excel_utils.get_column_code_by_index(colIndex)
            cell = ws[columnName + str(rowIndex)]


            link = None
            if None is not cell.hyperlink:
                link = cell.hyperlink.target if None is not cell.hyperlink.target and "" != cell.hyperlink.target else None

            if content['maxBeginIndex'] < 1:
                # 还没有标题，需要分析标题对齐位置

                info = {'content': value, 'horizontal': cell.alignment.horizontal, 'link': link}

                row["data"].append(info)
                pass
            else:
                # 已有标题
                row["data"].append({
                    'content': value,
                    'link': link
                })


        row['data'] = row['data'][:(row['endIndex'] - row['beginIndex'] + 1)]
        content['maxBeginIndex'] = row['beginIndex'] if (content['maxBeginIndex'] < 1 or row['beginIndex'] < content['maxBeginIndex']) else content['maxBeginIndex']
        content['maxEndIndex'] = row['endIndex'] if row['endIndex'] > content['maxEndIndex'] else content['maxEndIndex']


        if row['beginIndex'] < 1 and content['title'] is None:
            continue
        elif content['title'] is None:
            content['title'] = row
        else:
            content['data'].append(row)

    return content


def get_markdown_horizontal(horizontal):
    ''' 获得列对齐策略 '''
    if None == horizontal or "" == horizontal:
        return  ':--'
    elif horizontal == 'left':
        return ':--'
    elif horizontal == 'center':
        return ':--:'
    else:
        return '--:'

def format_markdown_content(content):
    ''' 格式化markdown表格内容 '''
    c = str(content.get('content', ''))
    link = content.get('link', None)
    c = c.replace('&', '&amp;')
    c = c.replace('"', '&quot;')
    c = c.replace('|', '&vert;') # &brvbar;  &vert;
    c = c.replace('<', '&lt;')
    c = c.replace('>', '&gt;')
    c = c.replace(' ', '&nbsp;')
    c = c.replace('\r\n', '<br />')
    c = c.replace('\n', '<br />')

    if link is None or '' == link:
        return c
    else:
        return "[%s](%s)" % (c, link)


def build_content(content, mode):
    ''' 构造保存文件内容 '''
    if mode == 'md':
        md = ''
        linesep = '\n'
        sep = '|'
        maxBeginIndex = content['maxBeginIndex']
        maxEndIndex = content['maxEndIndex']

        if content['maxBeginIndex'] < 1:
            return md

        title = sep
        horizontal = sep
        beginIndex = content['title']['beginIndex']

        for i in range(maxBeginIndex, maxEndIndex + 1):
            if i < beginIndex:
                title += sep
                horizontal += get_markdown_horizontal('') + sep
                continue
            idx = i - beginIndex
            if idx < len(content['title']['data']):
                title += format_markdown_content(content['title']['data'][idx])
                horizontal += get_markdown_horizontal(content['title']['data'][idx]['horizontal'])
            title += sep
            horizontal += sep

        md = title + linesep + horizontal + linesep

        for row in content['data']:
            beginIndex = row['beginIndex']
            c = sep
            for i in range(maxBeginIndex, maxEndIndex + 1):
                if i < beginIndex:
                    title += sep
                    continue
                idx = i - beginIndex
                if idx < len(row['data']):
                    c += format_markdown_content(row['data'][idx])
                c += sep
            md += c + linesep
        return md
    else:
        return ''



def save_excel_info(contents, path, mode):
    '''保存转换的excel信息'''

    # 创建文件夹
    file_utils.make_folders(path)

    # 保存文件
    for k, content in contents.items():
        c = build_content(content, mode)
        filePath = os.path.join(path, k+'.md')
        file_utils.write_file(filePath, c)


def format_clipboard_content(c):
    ''' 转义剪贴板中单元格内容，为了适配单元格内容中有换行符前后默认会加引号 '''
    if len(c) >= 6:
        if c[:3] == '"""' and c[-3:] == '"""':
            return c[3:-3]
    if len(c) >= 3:
        if c[:1] == '"' and c[-1:] == '"':
            return c[1:-1]
    return c


def clipboard_to_markdown(cb):
    ''' 将剪贴板中的excel表格内容转义成markdown内容 '''
    content = ''
    index = 0
    for line in cb.split('\r\n'):
        if line.strip() == '':
            continue

        ls = line.replace('\n', '<br />').split('\t')
        for l in ls:
            content += '| ' + format_clipboard_content(l) +' '
        content += '|\n'

        if index == 0:
            # 补头
            for l in ls:
                content += '|:--'
            content += '|\n'
        index += 1
    pyperclip.copy(content)
    return 0


def run(args):

    cb = pyperclip.paste()
    if None == args.excel:
        if cb == None or cb == '':
            print_msg('file path or clipboard is none.')
            return ResultCode.PARAMS_ERROR
        else:
            result = clipboard_to_markdown(cb)
            if True == args.paste:
                pyperclip.paste()
            return result

    if not os.path.isfile(args.excel):
        print_msg('file path is none.')
        return ResultCode.EXCEL_FILE_NOT_EXIST

    sheets = []
    if args.sheets is not None and args.sheets != '':
        sheets = args.sheets.split(',')

    contents = read_excel(args.excel, sheets)

    target = args.target
    if None == args.target or "" == args.target:
        target = os.path.join('./', file_utils.get_file_name_no_suffix(args.excel))

    mode = args.mode
    if mode != "md":
        mode = "md"
    save_excel_info(contents, target, args.mode)

    return 0













#-*- encoding: utf-8 -*-

#!/usr/bin/env python


import argparse

import soc_excel_convert._internal.soc_excel_convert as sec



def read_args():
    ''' read command line args '''
    parser = argparse.ArgumentParser()
    parser.add_argument("-e", "--excel", help="Excel file path.If it is empty, it will get excel data from the clipboard by default.")
    parser.add_argument("-m", "--mode", help = "Convert format.support:md (markdown); default md.", action="store_true", default = "md")
    parser.add_argument("-t", "--target", help="Target file path. Default ./{FileName}/ ", action="store_true", default = "")
    parser.add_argument("-s", "--sheets", help="Convert excel sheet name. Default all sheets. Multiple comma separated.", action="store_true",
                        default="")
    parser.add_argument("-p", "--paste", help = "Output clipboard contents.Default not output.",  action="store_true", default = False)

    return parser.parse_args()




def main(args=None):

    args = read_args()

    sec.run(args)




#-*- encoding: utf-8 -*-

import json
from datetime import date, datetime


class JsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime('%Y-%m-%d')
        else:
            return json.JSONEncoder.default(self, obj)

def json_encode(obj):
    return json.dumps(obj, cls = JsonEncoder)


def json_decode(value):
    return json.loads(value)


#-*- encoding: utf-8 -*-

import os
import io

from setuptools import find_packages, setup


if os.path.exists("requirements.txt"):
    with open("README.md", "r", encoding='UTF-8') as fh:
        long_description = fh.read()
else:
    long_description = ""

if os.path.exists("requirements.txt"):
    with open("requirements.txt", "r", encoding='UTF-8') as fh:
        install_requires = fh.read().split("\n")
else:
    install_requires = []


setup(
    name="soc-excel-convert",
    version="1.0.0",
    author="Tree",
    author_email="tree@ejyi.com",
    description="Convert Excel to markdown ....",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/treeyh/soc-excel-convert",
    keywords=["soc", "excel", "convert", "markdown"],
    # packages=['src'],
    package_dir={"": "src"},
    packages=find_packages(
        where="src",
        exclude=["docs"],
    ),
    license='Apache License 2.0',
    classifiers=[
        "Programming Language :: Python :: 3",
        'License :: OSI Approved :: Apache Software License',
        "Operating System :: OS Independent",
    ],

    entry_points={
        "console_scripts": [
            "soc-excel-convert=soc_excel_convert._internal:main",
        ],
    },

    install_requires=install_requires,
    zip_safe=False,
    python_requires='>=3',
)